#ifndef __SEKRIT_LAB_H__2014
#define __SEKRIT_LAB_H__2014

/*! \brief Perform one of the tests.
 *
 * For debugging only!
 *
 * \return 0 = no valid test found, otherwise 1
 */
int perform_a_sekrit_test(char *arg);

#endif
